package org.rcisoft.business.crawler.api;

import com.fasterxml.jackson.core.type.TypeReference;
import lombok.SneakyThrows;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.business.crawler.enums.WxResultStatus;
import org.rcisoft.business.crawler.exceptions.WxApiException;
import org.rcisoft.business.crawler.model.Article;
import org.rcisoft.business.crawler.model.BizData;
import org.rcisoft.business.crawler.model.WxResultBody;
import org.rcisoft.business.crawler.okhttp.MyCookieStore;
import org.rcisoft.business.crawler.utils.HttpUtils;
import org.rcisoft.business.crawler.utils.JsonUtils;

import java.io.InputStream;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 微信api封装
 * @author gxk
 */
public class WeiXinApi {

    /**接口地址**/
    public static final Map<String, String> URL_MAP = new HashMap<>();

    static {
        //初始化登录cookie
        URL_MAP.put("startLogin", "https://mp.weixin.qq.com/cgi-bin/bizlogin?action=startlogin");

        //二维码图片接口
        URL_MAP.put("getqrcode", "https://mp.weixin.qq.com/cgi-bin/scanloginqrcode");

        //二维码扫描状态接口
        URL_MAP.put("qrcodeAsk", "https://mp.weixin.qq.com/cgi-bin/scanloginqrcode");

        //扫码确认后，登录接口
        URL_MAP.put("bizlogin", "https://mp.weixin.qq.com/cgi-bin/bizlogin?action=login");

        //扫码确认后，登录接口
        URL_MAP.put("searchbiz", "https://mp.weixin.qq.com/cgi-bin/searchbiz");

        //扫码确认后，登录接口
//        URL_MAP.put("findListEx", "https://mp.weixin.qq.com/cgi-bin/appmsgpublish");
        URL_MAP.put("findListEx", "https://mp.weixin.qq.com/cgi-bin/appmsg");
    }

    /**
     * POST请求开始登录接口，初始化cookie
     * @param sessionid 时间戳，加两位随机数
     * @return
     */
    public static WxResultBody startLogin(String sessionid) {
        if (StringUtils.isBlank(sessionid)) {
            sessionid = "" + System.currentTimeMillis() + (int)(Math.random()*100);
        }

        Map<String, String> params = new HashMap<>(8);
        params.put("sessionid", sessionid);
        params.put("userlang", "zh_CN");
        params.put("redirect_url", "");
        params.put("login_type", "3");
        params.put("token", "");
        params.put("lang", "zh_CN");
        params.put("f", "json");
        params.put("ajax", "1");

        WxResultBody wxResultBody = parseWxResultBody(HttpUtils.doPost(URL_MAP.get("startLogin"), params));

        return wxResultBody;
    }

    /**
     * 获取二维码图片 字节输出流
     * @return
     */
    public static InputStream getQRCode() {
        Map<String, String> params = new HashMap<>(2);
        params.put("action", "getqrcode");
        params.put("random", System.currentTimeMillis() + "");
        InputStream inputStream = HttpUtils.doGetStream(URL_MAP.get("getqrcode"), params);

        return inputStream;
    }


    /**
     * GET 轮询二维码状态接口
     * @return
     */
    public static WxResultBody askQRCode() {
        Map<String, String> params = new HashMap<>(5);
        params.put("action", "ask");
        params.put("token", "");
        params.put("lang", "zh_CN");
        params.put("f", "json");
        params.put("ajax", "1");

        WxResultBody wxResultBody = parseWxResultBody(HttpUtils.doGet(URL_MAP.get("qrcodeAsk"), params));

        return wxResultBody;
    }

    /**
     * 扫码确认后，登录接口
     * @return
     */
    public static WxResultBody bizlogin() {
        Map<String, String> params = new HashMap<>(10);
        params.put("userlang", "zh_CN");
        params.put("redirect_url", "");
        params.put("cookie_forbidden", "0");
        params.put("cookie_cleaned", "0");
        params.put("plugin_used", "0");
        params.put("login_type", "3");
        params.put("token", "");
        params.put("lang", "zh_CN");
        params.put("f", "json");
        params.put("ajax", "1");

        WxResultBody wxResultBody = parseWxResultBody(HttpUtils.doPost(URL_MAP.get("bizlogin"), params));

        return wxResultBody;
    }


    /**
     * 搜索公众号
     * @return
     */
    public static WxResultBody<List<BizData>> searchBiz(String keyword){
        Map<String, String> params = new HashMap<>(10);
        params.put("action", "search_biz");
        params.put("begin", "0");
        params.put("count", "5");
        params.put("query", keyword);
        params.put("token", MyCookieStore.getToken());
        params.put("lang", "zh_CN");
        params.put("f", "json");
        params.put("ajax", "1");

        WxResultBody<List<BizData>> wxResultBody = parseWxResultBody(HttpUtils.doGet(URL_MAP.get("searchbiz"), params),
                new TypeReference<WxResultBody<List<BizData>>>() {}
        );

        return wxResultBody;
    }

    /**
     * 搜索公众号的文章
     * @return
     */
    public static WxResultBody<List<Article>> findExList(String fakeid){
        Map<String, String> params = new HashMap<>(10);
        params.put("action", "list_ex");
        params.put("begin", "0");
        params.put("count", "20");
        params.put("fakeid", fakeid);
        params.put("token", MyCookieStore.getToken());
        params.put("type", "9");
        params.put("query", "");
        params.put("lang", "zh_CN");
        params.put("f", "json");
        params.put("ajax", "1");

        WxResultBody<List<Article>> wxResultBody = parseWxResultBody(HttpUtils.doGet(URL_MAP.get("findListEx"), params),
                new TypeReference<WxResultBody<List<Article>>>() {}
        );

        return wxResultBody;
    }


    /**
     * 搜索公众号的文章（分页版本）
     * @return
     */
    public static WxResultBody<List<Article>> findExList(int begin, int count, String fakeId){
        Map<String, String> params = new HashMap<>(10);
        params.put("action", "list_ex");
        params.put("begin", String.valueOf(begin));
        params.put("count", String.valueOf(count));
        params.put("fakeid", fakeId);
        params.put("token", MyCookieStore.getToken());
        params.put("type", "9");
        params.put("query", "");
        params.put("lang", "zh_CN");
        params.put("f", "json");
        params.put("ajax", "1");

        WxResultBody<List<Article>> wxResultBody = parseWxResultBody(HttpUtils.doGet(URL_MAP.get("findListEx"), params),
                new TypeReference<WxResultBody<List<Article>>>() {}
        );

        return wxResultBody;
    }

    /**
     * 根据指定条数获取公众号文章
     * @param num
     * @param fakeId
     * @return
     */
    public static List<Article> getArticleList(int num, String fakeId) {
        List<Article> articleList = new ArrayList<>();
        int initialNum = 0;
        int loopNum = 0;
        int begin = 0;
        int count = 5;

        while (initialNum < num) {
            if (loopNum > 0) {
                begin += 5;
                count += 5;
            }
            WxResultBody<List<Article>> exList = findExList(begin, count, fakeId);
            List<Article> appMsgList = exList.getApp_msg_list();
            loopNum++;
            initialNum += appMsgList.size();
            articleList.addAll(appMsgList);
            delayedSleep();
        }

        List<Article> collect = articleList.stream().limit(num).collect(Collectors.toList());
        return collect;
    }

    /**
     * 延迟睡眠
     */
    @SneakyThrows
    public static void delayedSleep() {
        Random random = new Random();
        int minDelay = 200;
        int maxDelay = 500;

        // 生成一个随机延迟时间，范围在minDelay和maxDelay之间
        int randomDelay = minDelay + random.nextInt(maxDelay - minDelay + 1);
        Thread.sleep(randomDelay);
    }

    /**
     * 转成java bean
     * @param jsonRes json结果字符串
     * @return
     */
    public static WxResultBody parseWxResultBody(String jsonRes) {
        WxResultBody wxResultBody = JsonUtils.jsonToObj(jsonRes, WxResultBody.class);

        checkIsSuccess(wxResultBody);

        return wxResultBody;
    }

    /**
     * 转成java bean
     * @param jsonRes json结果字符串
     * @return
     */
    public static <T> T parseWxResultBody(String jsonRes, TypeReference<T> typeReference) {
        T wxResultBody = JsonUtils.jsonToObj(jsonRes, typeReference);
        checkIsSuccess((WxResultBody) wxResultBody);
        return wxResultBody;
    }

    public static void checkIsSuccess(WxResultBody wxResultBody) {
        // 判断是否请求成功
        if (wxResultBody == null) {
            throw new WxApiException(WxResultStatus.FAIL_NULL_RES);
        }

        if (wxResultBody.getBase_resp().getRet() == null
                || wxResultBody.getBase_resp().getRet() != 0) {
            throw new WxApiException(WxResultStatus.FAIL_STATUS);
        }
    }

}
